#include <iostream>
#include <vector>
#include <queue>
using namespace std;


// +oo
#define INFTY 99999999

// Bunch of macros to save some writing.
#define ii   pair<int, int>
#define vi   vector<int>
#define vii  vector<ii>
#define vvi  vector<vi>
#define vvii vector<vii>


// Operator used for comparison within the priority queue
// holding pair<int, int> values representing a node and
// its distance from the start node.
class cmpii
{
public:
	// The call operator needs to be overloaded in order
	// to customize priority_queue. Return true, if x
	// has lower priority than y.
    bool operator()(const ii& x, const ii& y)
    {
        return x.second > y.second;
    }
};


// Dijkstra's shortest path algorithm.
void dijkstra(vvii &G, vi &distances, int S, int D)
{
	// Indicator of nodes which shortest path
	// distances were determined.
	// XXX: Do we need such an indicator array?
	vector<bool> found(G.size(), false);

	// Upper bounds on distances from starting
	// node S for all vertices.
	distances.resize(G.size());

	// Initialize the distances.
	fill(distances.begin(), distances.end(), INFTY);

	// We definitely know this :).
	distances[S] = 0;

	// Priority queue holding the estimates of
	// the shortest paths from node S to every
	// (so far) visited node.
	priority_queue<ii, vii, cmpii> Q;

	// Push the start node S into the queue.
	Q.push(ii(S, distances[S]));

	while (!Q.empty())
	{
		int u = Q.top().first;
		Q.pop();

		// Continue if we already found the shortest
		// path for this node.
		// XXX: Why we might use something like this?
		//      Is it necessary?
		if (found[u])
			continue;

		found[u] = true;

		// Did we found the shortest path to
		// the destination node D?
		// XXX: It is not obvious (or is it?) that we
		//      can afford to stop the algorithm like
		//      this.
		if (u == D)
			break;

		// Relax (re-estimate) the shortest paths to all
		// neighboring nodes of node u.
		for (vii::iterator e = G[u].begin(); e != G[u].end(); ++e)
		{
			// Trying an edge e = (u, v) with length w.
			int v = e->first;
			int w = e->second;

			// Check whether we can get to node v through
			// node u using shorter distance than what is
			// the current estimate for u.
			if (!found[v] && distances[v] > distances[u] + w)
			{
				distances[v] = distances[u] + w;
				Q.push(ii(v, distances[v]));
			}
		}
	}
}

vvii reverse_graph(vvii &G)
{
	vvii reversedG(G.size());

	for (int u = 0; u < G.size(); ++u)
	{
		for (vii::iterator e = G[u].begin(); e != G[u].end(); ++e)
		{
			int v = e->first;
			int w = e->second;

			reversedG[v].push_back(ii(u, w));
		}
	}

	return reversedG;
}

// Remove every edge from the specified graph G that lies on any
// shortest path from node S to node D.
void remove_edges(vvii &G, vi &distances, int S, int D)
{
	// Queue only for nodes laying on a shortest path.
	queue<int> Q;

	vector<bool> pushed(G.size(), false);

	// We know that S is certainly on a shortest path.
	Q.push(S);
	pushed[S] = true;

	// While the queue is not empty...
	while (!Q.empty())
	{
		// ... get a node from it ...
		int u = Q.front();
		Q.pop();

		int i = 0;

		// ... and check for each of its outgoing edges ...
		while (i < G[u].size())
		{
			int v = G[u][i].first;
			int w = G[u][i].second;

			// ... whether it lies on a shortest path.
			if (distances[u] == distances[v] + w)
			{
				// Removing the edge.
				G[u][i] = G[u][G[u].size() - 1];
				G[u].pop_back();

				// We know that node v lies on a shortest
				// path the edge is a part of, so unless
				// it has not been done so, we push it
				// into the queue.
				if (!pushed[v] && v != D)
				{
					Q.push(v);
					pushed[v] = true;
				}
			}
			else
				++i;
		}
	}
}

int main(void)
{
	int N, M, S, D, U, V, P;

	// Make cout and cin faster if you prefer
	// them over their C alternatives.
	ios::sync_with_stdio(false);

	// Read the graph size.
	while (cin >> N >> M && N != 0)
	{
		// Read the start and destination node.
		cin >> S >> D;

		// Adjacency list representation
		// of the graph.
		vvii G(N);

		// Read the graph specification.
		for (int i = 0; i < M; ++i)
		{
			cin >> U >> V >> P;

			// Add edge from U to V.
			G[U].push_back(ii(V, P));
		}

		vector<int> distances;

		// Run the shortest path algorithm.
		dijkstra(G, distances, S, D);

		// We do not need to worry about the almost
		// shortest path when no path existed from S
		// to D in the first place.
		if (distances[D] == INFTY)
		{
			cout << -1 << endl;
			continue;
		}

		// Reverse the edges in the graph.
		G = reverse_graph(G);

		// Remove edges laying on any shortest path.
		remove_edges(G, distances, D, S);

		// Find the shortest path in the reversed
		// graph without the shortest path edges,
		// which gives us a almost shortest path
		// length (if such path exists).
		dijkstra(G, distances, D, S);

		cout << (distances[S] != INFTY ? distances[S] : -1) << endl;
	}

	return 0;
}